# ---------------------------------------------------------------------------
# landcover.py Created on: 2016-08-31 16:01:12.00000 

# Description: classify the picture took in the field to three classes:
# green vegetation, dead materials, and bare soil

#author: Dandan Xu, Nanjing Forestry University
# ---------------------------------------------------------------------------

# Import arcpy module
import arcpy
import arcinfo 
import os
import arcgisscripting
import types
import sys
import locale
import math

# get the input argument values
inRaster = arcpy.GetParameterAsText(0)

#define the constant parameters (default: 60)
SepG1 = arcpy.GetParameter(1) 
if SepG1 == '#' or not SepG1:
    SepG1 = 60 # provide a default value if unspecified

SepG2 = arcpy.GetParameter(2) 
if SepG2 == '#' or not SepG2:
    SepG2 = 60 # provide a default value if unspecified
    
LowD = arcpy.GetParameter(3) 
if LowD == '#' or not LowD:
    LowD = 1 # provide a default value if unspecified

# define the output parameters
greenV = arcpy.GetParameterAsText(4) 
dead = arcpy.GetParameterAsText(5) 

#define the output table
tableall = arcpy.GetParameterAsText(6) 

#define the temporary file location
tempfiles = arcpy.GetParameterAsText(7)
if tempfiles == '#' or not tempfiles:
    tempfiles = "E:\\land_cover_project\\tempfiles.gdb" # provide a default value if unspecified
	
# load the arcgis extension
arcpy.CheckOutExtension("Spatial")
gp = arcgisscripting.create(10.5)
gp.OverwriteOutput = 1
gp.workspace = tempfiles
gp.scratchworkspace = tempfiles

#define workspace and scratch workspace
from arcpy.sa import *
from arcpy import env
env.overwriteOutput = True
env.workspace = tempfiles
env.scratchworkspace = tempfiles

#extract each band from the original tif file
Band_1 = arcpy.sa.Raster(inRaster+'/Band_1')
Band_2 = arcpy.sa.Raster(inRaster+'/Band_2')
Band_3 = arcpy.sa.Raster(inRaster+'/Band_3')

# standardize band 2 using the statistical information in band 2
# Process: band2_mean
b2meanResult = arcpy.GetRasterProperties_management(Band_2, "MEAN")
b2mean = b2meanResult.getOutput(0)
print(b2mean)

# Process: band2_std
b2stdResult = arcpy.GetRasterProperties_management(Band_2, "STD")
b2std = b2stdResult.getOutput(0)
print (b2std)

# Process: b2_stand
b2mean = float(b2mean)
b2std = float (b2std)
b2stand = ( Band_2 - b2mean ) / b2std
b2stand.save("b2stand")

# Process: b2_min
b2minResult = arcpy.GetRasterProperties_management(b2stand, "MINIMUM")
b2min = b2minResult.getOutput(0)
print (b2min)

# Process: b2_max
b2maxResult = arcpy.GetRasterProperties_management(b2stand, "MAXIMUM")
b2max = b2maxResult.getOutput(0)
print(b2max)

# Process: b2_normal
b2min = float (b2min)
b2max = float (b2max)
b2normal = (b2stand - b2min) * 1023 / (b2max - b2min)
b2normal.save("b2normal")

#standardize band 3 using the statistical information in band 3
# Process: b3_mean
b3meanResult = arcpy.GetRasterProperties_management(Band_3, "MEAN")
b3mean = b3meanResult.getOutput(0)
print(b3mean)

# Process: b3_std
b3stdResult = arcpy.GetRasterProperties_management(Band_3, "STD")
b3std = b3stdResult.getOutput(0)
print(b3std)

# Process: b3_stand
b3mean = float(b3mean)
b3std = float (b3std)
b3stand = (Band_3 - b3mean) / b3std
b3stand.save("b3stand")

# Process: b3_min
b3minResult = arcpy.GetRasterProperties_management(b3stand, "MINIMUM")
b3min = b3minResult.getOutput(0)
print (b3min)

# Process: b3_max
b3maxResult = arcpy.GetRasterProperties_management(b3stand, "MAXIMUM")
b3max = b3maxResult.getOutput(0)
print (b3max)

# Process: b3_normal
b3min = float (b3min)
b3max = float (b3max)
b3normal = (b3stand - b3min) * 1023 / (b3max - b3min)
b3normal.save("b3normal")

#standardize band 1 using the statistical information in band 1
# Process: band1_mean
b1meanResult = arcpy.GetRasterProperties_management(Band_1, "MEAN", "Band_1")
b1mean = b1meanResult.getOutput(0)
print (b1mean)
 
# Process: band1_std
b1stdResult = arcpy.GetRasterProperties_management(Band_1, "STD", "Band_1")
b1std = b1stdResult.getOutput(0)
print (b1std)
 
# Process: b1_stand
b1mean = float(b1mean)
b1std = float (b1std)
b1stand = (Band_1 - b1mean) / b1std
b1stand.save("b1stand")

# Process: b1_min
b1minResult = arcpy.GetRasterProperties_management(b1stand, "MINIMUM")
b1min = b1minResult.getOutput(0)
print (b1min)
 
# Process: b1_max
b1maxResult = arcpy.GetRasterProperties_management(b1stand, "MAXIMUM")
b1max = b1maxResult.getOutput(0)
print (b1max)
 
# Process: b1_normal
b1min = float (b1min)
b1max = float (b1max)
b1normal = (b1stand - b1min) * 1023 / (b1max - b1min)
b1normal.save("b1normal")

Red = b1normal
Green = b2normal
Blue = b3normal

#classification for green vegetation, dead materials, and bare soil
#set local variables
InTrueRaster = 1
inFalseConstant = 0 

# Process: classify green vegetation, dead materials and backgournd bare soil or ground information
#classify green vegetation
SepG1 = float(SepG1)
SepG2 = float(SepG2)
Raster1 = arcpy.sa.Con(((Green - Red) > SepG1) & ((Green - Blue) > SepG2), InTrueRaster, inFalseConstant)
arcpy.gp.SetNull_sa(Raster1, Raster1, greenV, "Value=0")

#maskout the classified green vegetation 
Raster2 = "Raster2"
arcpy.gp.SetNull_sa(Raster1, 1, Raster2, "Value=1")
Red1 = arcpy.gp.ExtractByMask(Red, Raster2)
Green1 = arcpy.gp.ExtractByMask(Green, Raster2)
Blue1 = arcpy.gp.ExtractByMask(Blue, Raster2)

#calculate mean value for each band when it did not masked out
b1mResult = arcpy.GetRasterProperties_management(Red, "MEAN")
b1m = b1mResult.getOutput(0) 
b1m = float (b1m)
print(b1m)

b2mResult = arcpy.GetRasterProperties_management(Green, "MEAN")
b2m = b2mResult.getOutput(0) 
b2m = float (b2m)
print(b2m)

b3mResult = arcpy.GetRasterProperties_management(Blue, "MEAN")
b3m = b3mResult.getOutput(0) 
b3m = float (b3m)
print(b3m)

# Process: classify_dead
Red1 = Raster(Red1)
Green1 = Raster(Green1)
Blue1 = Raster(Blue1)

LowD = float(LowD)
b1mc = LowD * b1m
b2mc = LowD * b2m
b3mc = LowD * b3m
dead0 = arcpy.sa.Con((Red1 > b1mc) & (Green1 > b2mc) & (Blue1 > b3mc), InTrueRaster, inFalseConstant)
arcpy.gp.SetNull_sa(dead0, dead0, dead, "Value=0")

# Process: calculate the attribute table with final parameters to correct LAI
allbound = arcpy.sa.Con((Red >= 0), InTrueRaster)

GreenT = arcpy.CopyRows_management(greenV)
DeadT = arcpy.CopyRows_management(dead)
allT = arcpy.CopyRows_management(allbound)

arcpy.AddField_management(GreenT, "type", "TEXT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
arcpy.AddField_management(DeadT, "type", "TEXT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
arcpy.AddField_management(allT, "type", "TEXT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")

arcpy.CalculateField_management(GreenT, "type", "'green_count'", "PYTHON", "")
arcpy.CalculateField_management(DeadT, "type", "'dead_count'", "PYTHON", "")
arcpy.CalculateField_management(allT, "type", "'total_count'", "PYTHON", "")

fieldMappings = arcpy.FieldMappings()
fieldMappings.addTable(GreenT)
fieldMappings.addTable(DeadT)
fieldMappings.addTable(allT)

fldMap_value = arcpy.FieldMap()
fldMap_value.addInputField(GreenT,"Value")
fldMap_value.addInputField(DeadT,"Value")
fldMap_value.addInputField(allT,"Value")
value = fldMap_value.outputField
value.name = "Value"
fldMap_value.outputField = value
fieldMappings.addFieldMap(fldMap_value)

fldMap_type = arcpy.FieldMap()
fldMap_type.addInputField(GreenT,"type")
fldMap_type.addInputField(DeadT,"type")
fldMap_type.addInputField(allT,"type")
type = fldMap_type.outputField
type.name = "type"
fldMap_value.outputField = type
fieldMappings.addFieldMap(fldMap_type)

fldMap_count = arcpy.FieldMap()
fldMap_count.addInputField(GreenT,"Count")
fldMap_count.addInputField(DeadT,"Count")
fldMap_count.addInputField(allT,"Count")
count = fldMap_count.outputField
count.name = "Count"
fldMap_value.outputField = count
fieldMappings.addFieldMap(fldMap_count)

mergeT = arcpy.CreateTable_management(tempfiles, "mergeT")
arcpy.Merge_management([GreenT, DeadT, allT], mergeT, fieldMappings)

arcpy.PivotTable_management(mergeT, "Value", "type", "Count", tableall)

arcpy.AddField_management(tableall, "dead_percent", "FLOAT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
arcpy.AddField_management(tableall, "green_percent", "FLOAT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")

arcpy.CalculateField_management(tableall,"dead_percent", "( !dead_count! / !total_count! ) *100", "PYTHON", "")
arcpy.CalculateField_management(tableall,"green_percent", "( !green_count! / !total_count!) * 100", "PYTHON", "")




